@@ -317,6 +317,13 @@ exsltCryptoCryptoApiRc4Decrypt (xmlXPathParserContextPtr ctxt,
 #define PLATFORM_MD5 GCRY_MD_MD5
 #define PLATFORM_SHA1 GCRY_MD_SHA1
 
+#ifdef HAVE_SYS_TYPES_H                                                        
+# include <sys/types.h>                                                        
+#endif                                                                         
+#ifdef HAVE_STDINT_H                                                           
+# include <stdint.h>                                                           
+#endif                                                                         
+  
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>		/* needed by gcrypt.h 4 Jul 04 */
 #endif
