@@ -52,6 +52,11 @@ const xmlChar *xsltExtMarker = (const xmlChar *) "Extension Element";
  *									*
  ************************************************************************/
 
+#ifdef XSLT_REFACTORED
+    /*
+    * Grammar checks are now performed in xslt.c.
+    */
+#else
 /**
  * xsltCheckTopLevelElement:
  * @style: the XSLT stylesheet
@@ -197,6 +202,7 @@ xsltCheckParentElement(xsltStylesheetPtr style, xmlNodePtr inst,
 		       inst->name);
     style->errors++;
 }
+#endif
 
 /************************************************************************
  *									*
@@ -350,7 +356,6 @@ xsltNewStylePreComp(xsltStylesheetPtr style, xsltStyleType type) {
 	case XSLT_FUNC_PARAM:	    
 	case XSLT_FUNC_VARIABLE:	    
 	case XSLT_FUNC_WHEN:
-	case XSLT_FUNC_OTHERWISE:
 	    break;
 	default:
 	if (cur->func == NULL) {
@@ -380,6 +385,8 @@ xsltFreeStylePreComp(xsltStylePreCompPtr comp) {
     * URGENT TODO: Implement destructors.
     */
     switch (comp->type) {
+	case XSLT_FUNC_LITERAL_RESULT_ELEMENT:
+	    break;
 	case XSLT_FUNC_COPY:
             break;
         case XSLT_FUNC_SORT: {
@@ -468,7 +475,12 @@ xsltFreeStylePreComp(xsltStylePreCompPtr comp) {
 		    xmlXPathFreeCompExpr(item->comp);
 	    }
 	    break;
-	case XSLT_FUNC_OTHERWISE:
+	case XSLT_FUNC_OTHERWISE:	    
+	case XSLT_FUNC_FALLBACK:
+	case XSLT_FUNC_MESSAGE:
+	case XSLT_FUNC_INCLUDE:
+	case XSLT_FUNC_ATTRSET:
+	
 	    break;
 	default:
 	    /* TODO: Raise error. */
@@ -518,7 +530,8 @@ xsltDocumentComp(xsltStylesheetPtr style, xmlNodePtr inst,
     * "output" in XSLT_SAXON_NAMESPACE
     * "write" XSLT_XALAN_NAMESPACE
     * "document" XSLT_XT_NAMESPACE
-    * "document" XSLT_NAMESPACE
+    * "document" XSLT_NAMESPACE (from the abandoned old working
+    *                            draft of XSLT 1.1)
     * (in libexslt/common.c)
     * "document" in EXSLT_COMMON_NAMESPACE
     */
@@ -544,6 +557,8 @@ xsltDocumentComp(xsltStylesheetPtr style, xmlNodePtr inst,
 	*   (http://icl.com/saxon)
 	* The @file is in no namespace; it is an AVT.
 	*   (http://www.computerwizards.com/saxon/doc/extensions.html#saxon:output)
+	*
+	* TODO: Do we need not to check the namespace here?
 	*/
 	filename = xsltEvalStaticAttrValueTemplate(style, inst,
 			 (const xmlChar *)"file",
@@ -557,6 +572,10 @@ xsltDocumentComp(xsltStylesheetPtr style, xmlNodePtr inst,
 	/*
 	* TODO: Is "filename need to be interpreted" meant to be a todo?
 	*   Where will be the filename of xalan:write be processed?
+	*
+	* TODO: Do we need not to check the namespace here?
+	*   The extension ns is "http://xml.apache.org/xalan/redirect".
+	*   See http://xml.apache.org/xalan-j/extensionslib.html.
 	*/
     } else if (xmlStrEqual(inst->name, (const xmlChar *) "document")) {
 	if (inst->ns != NULL) {
@@ -770,11 +789,17 @@ xsltCopyComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	comp->has_use = 1;
 }
 
+#ifdef XSLT_REFACTORED
+    /* Enable if ever needed for xsl:text. */
+#else
 /**
  * xsltTextComp:
  * @style: an XSLT compiled stylesheet
  * @inst:  the xslt text node
  *
+ * TODO: This function is obsolete, since xsl:text won't
+ *  be compiled, but removed from the tree.
+ *
  * Process the xslt text node on the source node
  */
 static void
@@ -807,13 +832,14 @@ xsltTextComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	if (xmlStrEqual(prop, (const xmlChar *)"yes")) {
 	    comp->noescape = 1;
 	} else if (!xmlStrEqual(prop,
-				(const xmlChar *)"no")){
+	    (const xmlChar *)"no")){
 	    xsltTransformError(NULL, style, inst,
-"xsl:text: disable-output-escaping allows only yes or no\n");
+		"xsl:text: disable-output-escaping allows only yes or no\n");
 	    if (style != NULL) style->warnings++;
 	}
     }
 }
+#endif /* else of XSLT_REFACTORED */
 
 /**
  * xsltElementComp:
@@ -852,31 +878,85 @@ xsltElementComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     inst->psvi = comp;
     comp->inst = inst;
 
+    /*
+    * Attribute "name".
+    */
+    /*
+    * TODO: Precompile the AVT. See bug #344894.
+    */
     comp->name = xsltEvalStaticAttrValueTemplate(style, inst,
-				 (const xmlChar *)"name",
-				 NULL, &comp->has_name);
-    if (comp->name != NULL) {
+	(const xmlChar *)"name", NULL, &comp->has_name);
+    if (! comp->has_name) {
+	xsltTransformError(NULL, style, inst,
+	    "xsl:element: The attribute 'name' is missing.\n");
+	style->errors++;
+	goto error;
+    }
+    /*
+    * Attribute "namespace".
+    */
+    /*
+    * TODO: Precompile the AVT. See bug #344894.
+    */
+    comp->ns = xsltEvalStaticAttrValueTemplate(style, inst,
+	(const xmlChar *)"namespace", NULL, &comp->has_ns);
+    
+    if (comp->name != NULL) {	
 	if (xmlValidateQName(comp->name, 0)) {
 	    xsltTransformError(NULL, style, inst,
-		    "xsl:element : invalid name\n");
-	    if (style != NULL) style->errors++;
-	}
-    }
-    comp->ns = xsltEvalStaticAttrValueTemplate(style, inst,
-			 (const xmlChar *)"namespace",
-			 NULL, &comp->has_ns);
-    if (comp->has_ns == 0) {
-	xmlNsPtr defaultNs;
+		"xsl:element: The value '%s' of the attribute 'name' is "
+		"not a valid QName.\n", comp->name);
+	    style->errors++;
+	} else {
+	    const xmlChar *prefix = NULL, *name;
 
-	defaultNs = xmlSearchNs(inst->doc, inst, NULL);
-	if (defaultNs != NULL) {
-	    comp->ns = xmlDictLookup(style->dict, defaultNs->href, -1);
-	    comp->has_ns = 1;
+	    name = xsltSplitQName(style->dict, comp->name, &prefix);
+	    if (comp->has_ns == 0) {	    
+		xmlNsPtr ns;
+
+		/*
+		* SPEC XSLT 1.0:
+		*  "If the namespace attribute is not present, then the QName is
+		*  expanded into an expanded-name using the namespace declarations
+		*  in effect for the xsl:element element, including any default
+		*  namespace declaration.
+		*/		
+		ns = xmlSearchNs(inst->doc, inst, prefix);
+		if (ns != NULL) {
+		    comp->ns = xmlDictLookup(style->dict, ns->href, -1);
+		    comp->has_ns = 1;
+#ifdef XSLT_REFACTORED
+		    comp->nsPrefix = prefix;
+		    comp->name = name;
+#endif
+		} else if (prefix != NULL) {
+		    xsltTransformError(NULL, style, inst,
+			"xsl:element: The prefixed QName '%s' "
+			"has no namespace binding in scope in the "
+			"stylesheet; this is an error, since the namespace was "
+			"not specified by the instruction itself.\n", comp->name);
+		    style->errors++;
+		}
+	    }	    
+	    if ((prefix != NULL) &&
+		(!xmlStrncasecmp(prefix, (xmlChar *)"xml", 3)))
+	    {
+		/*
+		* Mark is to be skipped.
+		*/
+		comp->has_name = 0;		
+	    }
 	}
-    }
+    }    
+    /*
+    * Attribute "use-attribute-sets",
+    */
     comp->use = xsltEvalStaticAttrValueTemplate(style, inst,
 		       (const xmlChar *)"use-attribute-sets",
 		       NULL, &comp->has_use);
+
+error:    
+    return;
 }
 
 /**
@@ -905,7 +985,8 @@ xsltAttributeComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	return;
 
 #ifdef XSLT_REFACTORED
-    comp = (xsltStyleItemAttributePtr) xsltNewStylePreComp(style, XSLT_FUNC_ATTRIBUTE);
+    comp = (xsltStyleItemAttributePtr) xsltNewStylePreComp(style,
+	XSLT_FUNC_ATTRIBUTE);
 #else
     comp = xsltNewStylePreComp(style, XSLT_FUNC_ATTRIBUTE);
 #endif
@@ -916,22 +997,93 @@ xsltAttributeComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     comp->inst = inst;
 
     /*
-     * TODO: more computation can be done there, especially namespace lookup
-     */
+    * Attribute "name".
+    */
+    /*
+    * TODO: Precompile the AVT. See bug #344894.
+    */
     comp->name = xsltEvalStaticAttrValueTemplate(style, inst,
 				 (const xmlChar *)"name",
 				 NULL, &comp->has_name);
+    if (! comp->has_name) {
+	xsltTransformError(NULL, style, inst,
+	    "XSLT-attribute: The attribute 'name' is missing.\n");
+	style->errors++;
+	return;
+    }    
+    /*
+    * Attribute "namespace".
+    */
+    /*
+    * TODO: Precompile the AVT. See bug #344894.
+    */
+    comp->ns = xsltEvalStaticAttrValueTemplate(style, inst,
+	(const xmlChar *)"namespace",
+	NULL, &comp->has_ns);
+
     if (comp->name != NULL) {
 	if (xmlValidateQName(comp->name, 0)) {
 	    xsltTransformError(NULL, style, inst,
-			    "xsl:attribute : invalid QName\n");
-	    if (style != NULL) style->errors++;
-	}
-    }
-    comp->ns = xsltEvalStaticAttrValueTemplate(style, inst,
-			 (const xmlChar *)"namespace",
-			 NULL, &comp->has_ns);
+		"xsl:attribute: The value '%s' of the attribute 'name' is "
+		"not a valid QName.\n", comp->name);
+	    style->errors++;
+	} else {
+	    const xmlChar *prefix = NULL, *name;
 
+	    name = xsltSplitQName(style->dict, comp->name, &prefix);
+	    if (prefix != NULL) {
+		if (comp->has_ns == 0) {
+		    xmlNsPtr ns;
+
+		    /*
+		    * SPEC XSLT 1.0:
+		    *  "If the namespace attribute is not present, then the
+		    *  QName is expanded into an expanded-name using the
+		    *  namespace declarations in effect for the xsl:element
+		    *  element, including any default namespace declaration.
+		    */	    	    
+		    ns = xmlSearchNs(inst->doc, inst, prefix);
+		    if (ns != NULL) {
+			comp->ns = xmlDictLookup(style->dict, ns->href, -1);
+			comp->has_ns = 1;
+#ifdef XSLT_REFACTORED
+			comp->nsPrefix = prefix;
+			comp->name = name;
+#endif
+		    } else {
+			xsltTransformError(NULL, style, inst,
+			    "xsl:attribute: The prefixed QName '%s' "
+			    "has no namespace binding in scope in the "
+			    "stylesheet; this is an error, since the "
+			    "namespace was not specified by the instruction "
+			    "itself.\n", comp->name);
+			style->errors++;
+		    }
+		}
+		if (!xmlStrncasecmp(prefix, (xmlChar *) "xmlns", 5)) {
+		    /*
+		    * SPEC XSLT 1.0:
+		    *  "It is an error if the string that results from
+		    *  instantiating the attribute value template is not a
+		    *  QName or is the string xmlns. An XSLT processor may
+		    *  signal the error; if it does not signal the error,
+		    *  it must recover by not adding the attribute to the
+		    *  result tree."
+		    *
+		    * Reject a prefix of "xmlns". Mark to be skipped.
+		    */
+		    comp->has_name = 0;
+		    
+#ifdef WITH_XSLT_DEBUG_PARSING
+		    xsltGenericDebug(xsltGenericDebugContext,
+			"xsltAttribute: xmlns prefix forbidden\n");
+#endif		    
+		    return;
+		}
+		
+	    }
+	}	
+    }
 }
 
 /**
@@ -1104,6 +1256,66 @@ xsltValueOfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     }
 }
 
+static void
+xsltGetQNameProperty(xsltStylesheetPtr style, xmlNodePtr inst,
+		     const xmlChar *propName,
+		     int mandatory,
+		     int *hasProp, const xmlChar **nsName,
+		     const xmlChar** localName)
+{
+    const xmlChar *prop;
+
+    if (nsName)
+	*nsName = NULL;
+    if (localName)
+	*localName = NULL;
+    if (hasProp)
+	*hasProp = 0;
+
+    prop = xsltGetCNsProp(style, inst, propName, XSLT_NAMESPACE);
+    if (prop == NULL) {
+	if (mandatory) {
+	    xsltTransformError(NULL, style, inst,
+		"The attribute '%s' is missing.\n", propName);
+	    style->errors++;
+	    return;
+	}
+    } else {
+        const xmlChar *URI;
+
+	if (xmlValidateQName(prop, 0)) {
+	    xsltTransformError(NULL, style, inst,
+		"The value '%s' of the attribute "
+		"'%s' is not a valid QName.\n", prop, propName);
+	    style->errors++;
+	    return;
+	} else {
+	    /*
+	    * @prop will be in the string dict afterwards, @URI not.
+	    */
+	    URI = xsltGetQNameURI2(style, inst, &prop);
+	    if (prop == NULL) {
+		style->errors++;
+	    } else {
+		*localName = prop;
+		if (hasProp)
+		    *hasProp = 1;
+		if (URI != NULL) {
+		    /*
+		    * Fixes bug #308441: Put the ns-name in the dict
+		    * in order to pointer compare names during XPath's
+		    * variable lookup.
+		    */
+		    if (nsName)
+			*nsName = xmlDictLookup(style->dict, URI, -1);
+		    /* comp->has_ns = 1; */
+		}
+	    }
+	}
+    }
+    return;
+}
+
 /**
  * xsltWithParamComp:
  * @style: an XSLT compiled stylesheet
@@ -1124,7 +1336,6 @@ xsltWithParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 #else
     xsltStylePreCompPtr comp;
 #endif
-    const xmlChar *prop;
 
     if ((style == NULL) || (inst == NULL))
 	return;
@@ -1141,45 +1352,30 @@ xsltWithParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     comp->inst = inst;
 
     /*
-     * The full namespace resolution can be done statically
-     */
-    prop = xsltGetCNsProp(style, inst, (const xmlChar *)"name", XSLT_NAMESPACE);
-    if (prop == NULL) {
-	xsltTransformError(NULL, style, inst,
-	     "xsl:with-param : name is missing\n");
-	if (style != NULL) style->errors++;
-    } else {
-        const xmlChar *URI;
-
-	URI = xsltGetQNameURI2(style, inst, &prop);
-	if (prop == NULL) {
-	    if (style != NULL) style->errors++;
-	} else {
-	    comp->name = prop;
-	    comp->has_name = 1;
-	    if (URI != NULL) {
-		comp->ns = xmlStrdup(URI);
-		comp->has_ns = 1;
-	    } else {
-		comp->has_ns = 0;
-	    }
-	}
-    }
-
+    * Attribute "name".
+    */
+    xsltGetQNameProperty(style, inst, BAD_CAST "name",
+	1, &(comp->has_name), &(comp->ns), &(comp->name));
+    if (comp->ns)
+	comp->has_ns = 1;
+    /*
+    * Attribute "select".
+    */
     comp->select = xsltGetCNsProp(style, inst, (const xmlChar *)"select",
 	                        XSLT_NAMESPACE);
     if (comp->select != NULL) {
 	comp->comp = xsltXPathCompile(style, comp->select);
 	if (comp->comp == NULL) {
 	    xsltTransformError(NULL, style, inst,
-		 "xsl:param : could not compile select expression '%s'\n",
-			     comp->select);
-	    if (style != NULL) style->errors++;
+		 "XSLT-with-param: Failed to compile select "
+		 "expression '%s'\n", comp->select);
+	    style->errors++;
 	}
 	if (inst->children != NULL) {
 	    xsltTransformError(NULL, style, inst,
-	    "xsl:param : content should be empty since select is present \n");
-	    if (style != NULL) style->warnings++;
+		"XSLT-with-param: The content should be empty since "
+		"the attribute select is present.\n");
+	    style->warnings++;
 	}
     }
 }
@@ -1245,14 +1441,14 @@ xsltNumberComp(xsltStylesheetPtr style, xmlNodePtr cur) {
 	    xsltTransformError(NULL, style, cur,
 			 "xsl:number : invalid value %s for level\n", prop);
 	    if (style != NULL) style->warnings++;
-	    xmlFree((void *)(prop));
 	}
     }
     
     prop = xsltGetCNsProp(style, cur, (const xmlChar *)"lang", XSLT_NAMESPACE);
     if (prop != NULL) {
+	    xsltTransformError(NULL, style, cur,
+		 "xsl:number : lang attribute not implemented\n");
 	XSLT_TODO; /* xsl:number lang attribute */
-	xmlFree((void *)prop);
     }
     
     prop = xsltGetCNsProp(style, cur, (const xmlChar *)"letter-value", XSLT_NAMESPACE);
@@ -1343,7 +1539,6 @@ xsltCallTemplateComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 #else
     xsltStylePreCompPtr comp;
 #endif
-    const xmlChar *prop;
 
     if ((style == NULL) || (inst == NULL))
 	return;
@@ -1361,31 +1556,12 @@ xsltCallTemplateComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     comp->inst = inst;
 
     /*
-     * The full template resolution can be done statically
+     * Attribute "name".
      */
-    prop = xsltGetCNsProp(style, inst, (const xmlChar *)"name", XSLT_NAMESPACE);
-    if (prop == NULL) {
-	xsltTransformError(NULL, style, inst,
-	     "xsl:call-template : name is missing\n");
-	if (style != NULL) style->errors++;
-    } else {
-        const xmlChar *URI;
-
-	URI = xsltGetQNameURI2(style, inst, &prop);
-	if (prop == NULL) {
-	    if (style != NULL) style->errors++;
-	} else {
-	    comp->name = prop;
-	    comp->has_name = 1;
-	    if (URI != NULL) {
-		comp->ns = URI;
-		comp->has_ns = 1;
-	    } else {
-		comp->has_ns = 0;
-	    }
-	}
-	comp->templ = NULL;
-    }
+    xsltGetQNameProperty(style, inst, BAD_CAST "name",
+	1, &(comp->has_name), &(comp->ns), &(comp->name));
+    if (comp->ns)
+	comp->has_ns = 1;
 }
 
 /**
@@ -1402,7 +1578,6 @@ xsltApplyTemplatesComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 #else
     xsltStylePreCompPtr comp;
 #endif
-    const xmlChar *prop;
 
     if ((style == NULL) || (inst == NULL))
 	return;
@@ -1420,36 +1595,24 @@ xsltApplyTemplatesComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     comp->inst = inst;
 
     /*
-     * Get mode if any
+     * Attribute "mode".
      */
-    prop = xsltGetCNsProp(style, inst, (const xmlChar *)"mode", XSLT_NAMESPACE);
-    if (prop != NULL) {
-        const xmlChar *URI;
-
-	URI = xsltGetQNameURI2(style, inst, &prop);
-	if (prop == NULL) {
-	    if (style != NULL) style->errors++;
-	} else {
-	    comp->mode = xmlDictLookup(style->dict, prop, -1);
-	    if (URI != NULL) {
-		comp->modeURI = xmlDictLookup(style->dict, URI, -1);
-	    } else {
-		comp->modeURI = NULL;
-	    }
-	}
-    }
-    comp->select = xsltGetCNsProp(style, inst, (const xmlChar *)"select",
-	                        XSLT_NAMESPACE);
+    xsltGetQNameProperty(style, inst, BAD_CAST "mode",
+	0, NULL, &(comp->modeURI), &(comp->mode));
+    /*
+    * Attribute "select".
+    */
+    comp->select = xsltGetCNsProp(style, inst, BAD_CAST "select",
+	XSLT_NAMESPACE);
     if (comp->select != NULL) {
 	comp->comp = xsltXPathCompile(style, comp->select);
 	if (comp->comp == NULL) {
 	    xsltTransformError(NULL, style, inst,
-     "xsl:apply-templates : could not compile select expression '%s'\n",
-			     comp->select);
-	    if (style != NULL) style->errors++;
+		"XSLT-apply-templates: could not compile select "
+		"expression '%s'\n", comp->select);
+	     style->errors++;
 	}
     }
-
     /* TODO: handle (or skip) the xsl:sort and xsl:with-param */
 }
 
@@ -1638,7 +1801,6 @@ xsltVariableComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 #else
     xsltStylePreCompPtr comp;
 #endif
-    const xmlChar *prop;
 
     if ((style == NULL) || (inst == NULL))
 	return;
@@ -1652,49 +1814,38 @@ xsltVariableComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
     if (comp == NULL)
 	return;
+
     inst->psvi = comp;
     comp->inst = inst;
-
     /*
      * The full template resolution can be done statically
      */
-    prop = xsltGetCNsProp(style, inst, (const xmlChar *)"name", XSLT_NAMESPACE);
-    if (prop == NULL) {
-	xsltTransformError(NULL, style, inst,
-	     "xsl:variable : name is missing\n");
-	if (style != NULL) style->errors++;
-    } else {
-        const xmlChar *URI;
-
-	URI = xsltGetQNameURI2(style, inst, &prop);
-	if (prop == NULL) {
-	    if (style != NULL) style->errors++;
-	} else {
-	    comp->name = prop;
-	    comp->has_name = 1;
-	    if (URI != NULL) {
-		comp->ns = xmlDictLookup(style->dict, URI, -1);
-		comp->has_ns = 1;
-	    } else {
-		comp->has_ns = 0;
-	    }
-	}
-    }
 
+    /*
+    * Attribute "name".
+    */
+    xsltGetQNameProperty(style, inst, BAD_CAST "name",
+	1, &(comp->has_name), &(comp->ns), &(comp->name));
+    if (comp->ns)
+	comp->has_ns = 1;    
+    /*
+    * Attribute "select".
+    */
     comp->select = xsltGetCNsProp(style, inst, (const xmlChar *)"select",
 	                        XSLT_NAMESPACE);
     if (comp->select != NULL) {
 	comp->comp = xsltXPathCompile(style, comp->select);
 	if (comp->comp == NULL) {
 	    xsltTransformError(NULL, style, inst,
-		 "xsl:variable : could not compile select expression '%s'\n",
-			     comp->select);
-	    if (style != NULL) style->errors++;
+		"XSLT-variable: Failed to compile the XPath expression '%s'.\n",
+		comp->select);
+	    style->errors++;
 	}
 	if (inst->children != NULL) {
 	    xsltTransformError(NULL, style, inst,
-	"xsl:variable : content should be empty since select is present \n");
-	    if (style != NULL) style->warnings++;
+		"XSLT-variable: The must be no child nodes, since the "
+		"attribute 'select' was specified.\n");
+	    style->errors++;
 	}
     }
 }
@@ -1713,7 +1864,6 @@ xsltParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 #else
     xsltStylePreCompPtr comp;
 #endif
-    const xmlChar *prop;
 
     if ((style == NULL) || (inst == NULL))
 	return;
@@ -1731,128 +1881,34 @@ xsltParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     comp->inst = inst;
 
     /*
-     * The full template resolution can be done statically
+     * Attribute "name".
      */
-    prop = xsltGetCNsProp(style, inst, (const xmlChar *)"name", XSLT_NAMESPACE);
-    if (prop == NULL) {
-	xsltTransformError(NULL, style, inst,
-	     "xsl:param : name is missing\n");
-	if (style != NULL) style->errors++;
-    } else {
-        const xmlChar *URI;
-
-	URI = xsltGetQNameURI2(style, inst, &prop);
-	if (prop == NULL) {
-	    if (style != NULL) style->errors++;
-	} else {
-	    comp->name = prop;
-	    comp->has_name = 1;
-	    if (URI != NULL) {
-		comp->ns = xmlStrdup(URI);
-		comp->has_ns = 1;
-	    } else {
-		comp->has_ns = 0;
-	    }
-	}
-    }
-
+    xsltGetQNameProperty(style, inst, BAD_CAST "name",
+	1, &(comp->has_name), &(comp->ns), &(comp->name));
+    if (comp->ns)
+	comp->has_ns = 1;
+    /*
+    * Attribute "select".
+    */
     comp->select = xsltGetCNsProp(style, inst, (const xmlChar *)"select",
 	                        XSLT_NAMESPACE);
     if (comp->select != NULL) {
 	comp->comp = xsltXPathCompile(style, comp->select);
 	if (comp->comp == NULL) {
 	    xsltTransformError(NULL, style, inst,
-		 "xsl:param : could not compile select expression '%s'\n",
-			     comp->select);
-	    if (style != NULL) style->errors++;
+		"XSLT-param: could not compile select expression '%s'.\n",
+		comp->select);
+	    style->errors++;
 	}
 	if (inst->children != NULL) {
 	    xsltTransformError(NULL, style, inst,
-	"xsl:param : content should be empty since select is present \n");
-	    if (style != NULL) style->warnings++;
+		"XSLT-param: The content should be empty since the "
+		"attribute 'select' is present.\n");
+	    style->warnings++;
 	}
     }
 }
 
-#ifdef XSLT_REFACTORED
-
-/*
-* xsltCompilerGetInScopeNSInfo:
-*
-* Create and store the list of in-scope namespaces for the given
-* node in the stylesheet. If there are no changes in the in-scope
-* namespaces then the last ns-info of the ancestor axis will be returned.
-* Compilation-time only.
-*
-* Returns the ns-info or NULL if there are no namespaces in scope.
-*/
-xsltNsListPtr
-xsltCompilerGetInScopeNSInfo(xsltCompilerCtxtPtr cctxt, xmlNodePtr node)
-{
-    xsltNsListPtr nsi = NULL;
-    xmlNsPtr *list = NULL;
-    /*
-    * Create a new ns-list for this position in the node-tree.
-    * xmlGetNsList() will return NULL, if there are no ns-decls in the
-    * tree. Note that the ns-decl for the XML namespace is not added
-    * to the resulting list; the XPath module handles the XML namespace
-    * internally.
-    */
-    list = xmlGetNsList(node->doc, node);
-    if (list == NULL)
-	return(NULL);
-    /*
-    * Initialize the list hold by the stylesheet.
-    */
-    if (cctxt->sheet->inScopeNamespaces == NULL) {
-	cctxt->sheet->inScopeNamespaces = xsltPointerListCreate();
-	if (cctxt->sheet->inScopeNamespaces == NULL) {	    
-	    xsltTransformError(NULL, cctxt->sheet, NULL,
-		"xsltCompilerGetInScopeNSInfo: malloc failed.\n");	    
-	    goto internal_err;	    
-	}
-    }
-    /*
-    * Create the info-structure.
-    */
-    nsi = (xsltNsListPtr) xmlMalloc(sizeof(xsltNsList));
-    if (nsi == NULL) {	
-	xsltTransformError(NULL, cctxt->sheet, NULL,
-	    "xsltCompilerGetInScopeNSInfo: malloc failed.\n");
-	goto internal_err;
-    }
-    memset(nsi, 0, sizeof(xsltNsList));
-    nsi->list = list;
-    /*
-    * Eval the number of ns-decls; this is used to speed up
-    * XPath-context initialization.
-    */
-    while (list[nsi->number] != NULL)
-	nsi->number++;
-    /*
-    * Store the ns-list in the stylesheet.
-    */
-    if (xsltPointerListAdd(
-	(xsltPointerListPtr)cctxt->sheet->inScopeNamespaces,
-	(void *) nsi) == -1)
-    {	
-	xmlFree(nsi);
-	nsi = NULL;
-	xsltTransformError(NULL, cctxt->sheet, NULL,
-	    "xsltCompilerGetInScopeNSInfo: failed to add ns-info.\n");
-	goto internal_err;
-    }     
-
-    return(nsi);
-
-internal_err:
-    if (list != NULL)
-	xmlFree(list);    
-    cctxt->sheet->errors++;
-    return(NULL);
-}
-#endif /* XSLT_REFACTORED */
-
 /************************************************************************
  *									*
  *		    Generic interface					*
@@ -1870,10 +1926,11 @@ xsltFreeStylePreComps(xsltStylesheetPtr style) {
     xsltElemPreCompPtr cur, next;
 
     if (style == NULL)
-	return;
+	return;        
+    
     cur = style->preComps;
     while (cur != NULL) {
-	next = cur->next;
+	next = cur->next;		
 	if (cur->type == XSLT_FUNC_EXTENSION)
 	    cur->free(cur);
 	else
@@ -1882,6 +1939,221 @@ xsltFreeStylePreComps(xsltStylesheetPtr style) {
     }
 }
 
+#ifdef XSLT_REFACTORED
+
+/**
+ * xsltStylePreCompute:
+ * @style:  the XSLT stylesheet
+ * @node:  the element in the XSLT namespace
+ *
+ * Precompute an XSLT element.
+ * This expects the type of the element to be already
+ * set in style->compCtxt->inode->type;
+ */
+void
+xsltStylePreCompute(xsltStylesheetPtr style, xmlNodePtr node) {
+    /*    
+    * The xsltXSLTElemMarker marker was set beforehand by
+    *  the parsing mechanism for all elements in the XSLT namespace.
+    */
+    if (style == NULL) {
+	if (node != NULL)
+	    node->psvi = NULL;
+	return;
+    }
+    if (node == NULL)
+	return;
+    if (! IS_XSLT_ELEM_FAST(node))
+	return;
+
+    node->psvi = NULL;
+    if (XSLT_CCTXT(style)->inode->type != 0) {
+	switch (XSLT_CCTXT(style)->inode->type) {
+	    case XSLT_FUNC_APPLYTEMPLATES:
+		xsltApplyTemplatesComp(style, node);
+		break;
+	    case XSLT_FUNC_WITHPARAM:			   
+		xsltWithParamComp(style, node);
+		break;
+	    case XSLT_FUNC_VALUEOF:	    
+		xsltValueOfComp(style, node);
+		break;
+	    case XSLT_FUNC_COPY:	    
+		xsltCopyComp(style, node);
+		break;
+	    case XSLT_FUNC_COPYOF:
+		xsltCopyOfComp(style, node);
+		break;
+	    case XSLT_FUNC_IF:	    
+		xsltIfComp(style, node);
+		break;
+	    case XSLT_FUNC_CHOOSE:	    
+		xsltChooseComp(style, node);
+		break;
+	    case XSLT_FUNC_WHEN:	    
+		xsltWhenComp(style, node);
+		break;
+	    case XSLT_FUNC_OTHERWISE:	    
+		/* NOP yet */
+		return;
+	    case XSLT_FUNC_FOREACH:	    
+		xsltForEachComp(style, node);
+		break;
+	    case XSLT_FUNC_APPLYIMPORTS:	    
+		xsltApplyImportsComp(style, node);
+		break;
+	    case XSLT_FUNC_ATTRIBUTE:	    
+		xsltAttributeComp(style, node);
+		break;
+	    case XSLT_FUNC_ELEMENT:	    
+		xsltElementComp(style, node);
+		break;
+	    case XSLT_FUNC_SORT:	    
+		xsltSortComp(style, node);
+		break;
+	    case XSLT_FUNC_COMMENT:	    
+		xsltCommentComp(style, node);
+		break;
+	    case XSLT_FUNC_NUMBER:	    
+		xsltNumberComp(style, node);
+		break;
+	    case XSLT_FUNC_PI:	    
+		xsltProcessingInstructionComp(style, node);
+		break;
+	    case XSLT_FUNC_CALLTEMPLATE:	    
+		xsltCallTemplateComp(style, node);
+		break;
+	    case XSLT_FUNC_PARAM:	    
+		xsltParamComp(style, node);
+		break;
+	    case XSLT_FUNC_VARIABLE:	    
+		xsltVariableComp(style, node);
+		break;
+	    case XSLT_FUNC_FALLBACK:	    
+		/* NOP yet */
+		return;
+	    case XSLT_FUNC_DOCUMENT:	    
+		/* The extra one */
+		node->psvi = (void *) xsltDocumentComp(style, node,
+		    (xsltTransformFunction) xsltDocumentElem);
+		break;
+	    case XSLT_FUNC_MESSAGE:
+		/* NOP yet */
+		return;
+	    default:
+		/*
+		* NOTE that xsl:text, xsl:template, xsl:stylesheet,
+		*  xsl:transform, xsl:import, xsl:include are not expected
+		*  to be handed over to this function.
+		*/
+		xsltTransformError(NULL, style, node,
+		    "Internal error: (xsltStylePreCompute) cannot handle "
+		    "the XSLT element '%s'.\n", node->name);
+		style->errors++;
+		return;
+	}
+    } else {
+	/*
+	* Fallback to string comparison.
+	*/	
+	if (IS_XSLT_NAME(node, "apply-templates")) {
+	    xsltApplyTemplatesComp(style, node);
+	} else if (IS_XSLT_NAME(node, "with-param")) {
+	    xsltWithParamComp(style, node);
+	} else if (IS_XSLT_NAME(node, "value-of")) {
+	    xsltValueOfComp(style, node);
+	} else if (IS_XSLT_NAME(node, "copy")) {
+	    xsltCopyComp(style, node);
+	} else if (IS_XSLT_NAME(node, "copy-of")) {
+	    xsltCopyOfComp(style, node);
+	} else if (IS_XSLT_NAME(node, "if")) {
+	    xsltIfComp(style, node);
+	} else if (IS_XSLT_NAME(node, "choose")) {
+	    xsltChooseComp(style, node);
+	} else if (IS_XSLT_NAME(node, "when")) {
+	    xsltWhenComp(style, node);	
+	} else if (IS_XSLT_NAME(node, "otherwise")) {
+	    /* NOP yet */
+	    return;
+	} else if (IS_XSLT_NAME(node, "for-each")) {
+	    xsltForEachComp(style, node);
+	} else if (IS_XSLT_NAME(node, "apply-imports")) {
+	    xsltApplyImportsComp(style, node);
+	} else if (IS_XSLT_NAME(node, "attribute")) {
+	    xsltAttributeComp(style, node);
+	} else if (IS_XSLT_NAME(node, "element")) {
+	    xsltElementComp(style, node);
+	} else if (IS_XSLT_NAME(node, "sort")) {
+	    xsltSortComp(style, node);
+	} else if (IS_XSLT_NAME(node, "comment")) {
+	    xsltCommentComp(style, node);
+	} else if (IS_XSLT_NAME(node, "number")) {
+	    xsltNumberComp(style, node);
+	} else if (IS_XSLT_NAME(node, "processing-instruction")) {
+	    xsltProcessingInstructionComp(style, node);
+	} else if (IS_XSLT_NAME(node, "call-template")) {
+	    xsltCallTemplateComp(style, node);
+	} else if (IS_XSLT_NAME(node, "param")) {
+	    xsltParamComp(style, node);
+	} else if (IS_XSLT_NAME(node, "variable")) {
+	    xsltVariableComp(style, node);
+	} else if (IS_XSLT_NAME(node, "fallback")) {
+	    /* NOP yet */
+	    return;
+	} else if (IS_XSLT_NAME(node, "document")) {
+	    /* The extra one */
+	    node->psvi = (void *) xsltDocumentComp(style, node,
+		(xsltTransformFunction) xsltDocumentElem);	
+	} else if (IS_XSLT_NAME(node, "output")) {
+	    /* Top-level */
+	    return;
+	} else if (IS_XSLT_NAME(node, "preserve-space")) {
+	    /* Top-level */
+	    return;
+	} else if (IS_XSLT_NAME(node, "strip-space")) {
+	    /* Top-level */
+	    return;	
+	} else if (IS_XSLT_NAME(node, "key")) {
+	    /* Top-level */
+	    return;
+	} else if (IS_XSLT_NAME(node, "message")) {
+	    return;
+	} else if (IS_XSLT_NAME(node, "attribute-set")) {
+	    /* Top-level */
+	    return;
+	} else if (IS_XSLT_NAME(node, "namespace-alias")) {
+	    /* Top-level */
+	    return;
+	} else if (IS_XSLT_NAME(node, "decimal-format")) {
+	    /* Top-level */
+	    return;
+	} else if (IS_XSLT_NAME(node, "include")) {
+	    /* Top-level */	    	    
+	} else {
+	    /*
+	    * NOTE that xsl:text, xsl:template, xsl:stylesheet,
+	    *  xsl:transform, xsl:import, xsl:include are not expected
+	    *  to be handed over to this function.
+	    */
+	    xsltTransformError(NULL, style, node,
+		"Internal error: (xsltStylePreCompute) cannot handle "
+		"the XSLT element '%s'.\n", node->name);
+		style->errors++;
+	    return;
+	}	
+    }
+    /*
+    * Assign the current list of in-scope namespaces to the
+    * item. This is needed for XPath expressions.
+    */
+    if (node->psvi != NULL) {
+	((xsltStylePreCompPtr) node->psvi)->inScopeNs =
+	    XSLT_CCTXT(style)->inode->inScopeNs;
+    }
+}
+
+#else
+
 /**
  * xsltStylePreCompute:
  * @style:  the XSLT stylesheet
@@ -1895,10 +2167,14 @@ xsltStylePreCompute(xsltStylesheetPtr style, xmlNodePtr inst) {
     * URGENT TODO: Normally inst->psvi Should never be reserved here,
     *   BUT: since if we include the same stylesheet from
     *   multiple imports, then the stylesheet will be parsed
-    *   again. We simply must not try to compute the stylesheet again.    
+    *   again. We simply must not try to compute the stylesheet again.
+    * TODO: Get to the point where we don't need to query the
+    *   namespace- and local-name of the node, but can evaluate this
+    *   using cctxt->style->inode->category;
     */
-    if (inst->psvi != NULL) 
-        return;
+    if (inst->psvi != NULL)
+	return;
+
     if (IS_XSLT_ELEM(inst)) {
 	xsltStylePreCompPtr cur;
 
@@ -1965,7 +2241,7 @@ xsltStylePreCompute(xsltStylesheetPtr style, xmlNodePtr inst) {
 	} else if (IS_XSLT_NAME(inst, "call-template")) {
 	    xsltCheckInstructionElement(style, inst);
 	    xsltCallTemplateComp(style, inst);
-	} else if (IS_XSLT_NAME(inst, "param")) {
+	} else if (IS_XSLT_NAME(inst, "param")) {	   
 	    if (xsltCheckTopLevelElement(style, inst, 0) == 0)
 	        xsltCheckInstructionElement(style, inst);
 	    xsltParamComp(style, inst);
@@ -2034,18 +2310,7 @@ xsltStylePreCompute(xsltStylesheetPtr style, xmlNodePtr inst) {
 	    if (style != NULL) style->warnings++;
 	}
 	
-	
 	cur = (xsltStylePreCompPtr) inst->psvi;
-
-#ifdef XSLT_REFACTORED
-	/*
-	* Assign the current list of in-scope namespaces to the
-	* item. This is needed for XPath expressions.
-	*/
-	if (cur != NULL) {
-	    cur->inScopeNS = XSLT_CCTXT(style)->inode->inScopeNS;	    	    
-	}
-#else
 	/*
 	* A ns-list is build for every XSLT item in the
 	* node-tree. This is needed for XPath expressions.
@@ -2060,7 +2325,6 @@ xsltStylePreCompute(xsltStylesheetPtr style, xmlNodePtr inst) {
 	    }
 	    cur->nsNr = i;
 	}
-#endif
     } else {
 	inst->psvi =
 	    (void *) xsltPreComputeExtModuleElement(style, inst);
@@ -2073,3 +2337,4 @@ xsltStylePreCompute(xsltStylesheetPtr style, xmlNodePtr inst) {
 	    inst->psvi = (void *) xsltExtMarker;
     }
 }
+#endif /* XSLT_REFACTORED */
