@@ -4,6 +4,7 @@
 #
 
 functions = {}
+enums = {} # { enumType: { enumConstant: enumValue } }
 
 import string
 
@@ -136,6 +137,9 @@ class docParser:
                     self.function_return_info = attrs['info']
                 if attrs.has_key('field'):
                     self.function_return_field = attrs['field']
+        elif tag == 'enum':
+            enum(attrs['type'],attrs['name'],attrs['value']) 
+        
 
 
     def end(self, tag):
@@ -166,10 +170,13 @@ class docParser:
                 
                 
 def function(name, desc, ret, args, file):
-    global functions
-
     functions[name] = (desc, ret, args, file)
 
+def enum(type, name, value):
+    if not enums.has_key(type):
+        enums[type] = {}
+    enums[type][name] = value 
+
 #######################################################################
 #
 #  Some filtering rukes to drop functions/types which should not
@@ -209,7 +216,7 @@ skipped_types = {
 py_types = {
     'void': (None, None, None, None, None),
     'int':  ('i', None, "int", "int", "libxml_"),
-    'long':  ('i', None, "int", "int", "libxml_"),
+    'long':  ('l', None, "long", "long", "libxml_"),
     'double':  ('d', None, "double", "double", "libxml_"),
     'unsigned int':  ('i', None, "int", "int", "libxml_"),
     'xmlChar':  ('c', None, "int", "int", "libxml_"),
@@ -406,10 +413,11 @@ def print_function_wrapper(name, output, export, include):
         return 1
 
     output.write("PyObject *\n")
-    output.write("libxslt_%s(ATTRIBUTE_UNUSED PyObject *self," % (name))
+    output.write("libxslt_%s(PyObject *self ATTRIBUTE_UNUSED," % (name))
+    output.write(" PyObject *args")
     if format == "":
-	output.write("ATTRIBUTE_UNUSED ")
-    output.write(" PyObject *args) {\n")
+	output.write(" ATTRIBUTE_UNUSED")
+    output.write(") {\n")
     if ret[0] != 'void':
         output.write("    PyObject *py_retval;\n")
     if c_return != "":
@@ -556,6 +564,8 @@ primary_classes = ["xpathParserContext", "xpathContext", "transformCtxt", "style
 classes_ancestor = {
     "xpathContext" : "libxml2.xpathContext",
     "xpathParserContext" : "libxml2.xpathParserContext",
+    "transformCtxt": "transformCtxtBase",
+    "stylesheet": "stylesheetBase",
 }
 classes_destructors = {
     "xpathContext" : "pass"
@@ -942,6 +952,17 @@ def buildWrappers():
 			classes.write("        return ret\n");
 		classes.write("\n");
 
+    #
+    # Generate enum constants
+    #
+    for type,enum in enums.items():
+        classes.write("# %s\n" % type)
+        items = enum.items()
+        items.sort(lambda i1,i2: cmp(long(i1[1]),long(i2[1])))
+        for name,value in items:
+            classes.write("%s = %s\n" % (name,value))
+        classes.write("\n"); 
+
     txt.close()
     classes.close()
 
