@@ -122,7 +122,7 @@ def usage(name = 'pyxsltproc'):
     print "\t--catalogs : use SGML catalogs from $SGML_CATALOG_FILES"
     print "\t             otherwise XML Catalogs starting from "
     print "\t         file:///etc/xml/catalog are activated by default"
-    print "\t--xinclude : do XInclude processing on document intput"
+    print "\t--xinclude : do XInclude processing on document input"
     print "\t--profile or --norman : dump profiling informations "
     print "\nProject libxslt home page: http://xmlsoft.org/XSLT/"
     print "To report bugs and get help: http://xmlsoft.org/XSLT/bugs.html"
