@@ -26,10 +26,8 @@ var versionFile = ".\\config.msvc";
    is identical to the first. */
 var optsFileInXslt = srcDirXslt + "\\xsltconfig.h.in";
 var optsFileXslt = srcDirXslt + "\\xsltconfig.h";
-var optsFileXslt2 = srcDirXslt + "\\xsltwin32config.h";
 var optsFileInExslt = srcDirExslt + "\\exsltconfig.h.in";
 var optsFileExslt = srcDirExslt + "\\exsltconfig.h";
-var optsFileExslt2 = srcDirExslt + "\\exsltwin32config.h";
 /* Version strings for the binary distribution. Will be filled later 
    in the code. */
 var verMajorXslt;
@@ -53,6 +51,7 @@ var withModules = false;
 var dirSep = "\\";
 var compiler = "msvc";
 var cruntime = "/MD";
+var vcmanifest = false;
 var buildDebug = 0;
 var buildStatic = 0;
 var buildPrefix = ".";
@@ -110,6 +109,7 @@ function usage()
 	txt += "\nWin32 build options, default value given in parentheses:\n\n";
 	txt += "  compiler:   Compiler to be used [msvc|mingw] (" + compiler + ")\n";
 	txt += "  cruntime:   C-runtime compiler option (only msvc) (" + cruntime + ")\n";
+	txt += "  vcmanifest: Embed VC manifest (only msvc) (" + (vcmanifest? "yes" : "no") + ")\n";
 	txt += "  debug:      Build unoptimised debug executables (" + (buildDebug? "yes" : "no")  + ")\n";
 	txt += "  static:     Link xsltproc statically to libxslt (" + (buildStatic? "yes" : "no")  + ")\n";
 	txt += "              Note: automatically enabled if cruntime is not /MD or /MDd\n";
@@ -203,6 +203,7 @@ function discoverVersion()
 		vf.WriteLine("INCLUDE=$(INCLUDE);" + buildInclude);
 		vf.WriteLine("LIB=$(LIB);" + buildLib);
 		vf.WriteLine("CRUNTIME=" + cruntime);
+		vf.WriteLine("VCMANIFEST=" + (vcmanifest? "1" : "0"));
 	} else if (compiler == "mingw") {
 		vf.WriteLine("INCLUDE+=;" + buildInclude);
 		vf.WriteLine("LIB+=;" + buildLib);
@@ -246,7 +247,6 @@ function configureXslt()
 	}
 	ofi.Close();
 	of.Close();
-	fso.CopyFile(optsFileXslt, optsFileXslt2, true);
 }
 
 /* Configures libexslt. This one will generate exsltconfig.h from exsltconfig.h.in
@@ -277,7 +277,6 @@ function configureExslt()
 	}
 	ofi.Close();
 	of.Close();
-	fso.CopyFile(optsFileExslt, optsFileExslt2, true);
 }
 
 /* Creates the readme file for the binary distribution of 'bname', for the
@@ -348,6 +347,8 @@ for (i = 0; (i < WScript.Arguments.length) && (error == 0); i++) {
 			compiler = arg.substring(opt.length + 1, arg.length);
  		else if (opt == "cruntime")
  			cruntime = arg.substring(opt.length + 1, arg.length);
+		else if (opt == "vcmanifest")
+			vcmanifest = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "static")
 			buildStatic = strToBool(arg.substring(opt.length + 1, arg.length));
 		else if (opt == "prefix")
@@ -482,6 +483,7 @@ txtOut += "-------------------------\n";
 txtOut += "          Compiler: " + compiler + "\n";
 if (compiler == "msvc")
 	txtOut += "  C-Runtime option: " + cruntime + "\n";
+	txtOut += "    Embed Manifest: " + boolToStr(vcmanifest) + "\n";
 txtOut += "     Debug symbols: " + boolToStr(buildDebug) + "\n";
 txtOut += "   Static xsltproc: " + boolToStr(buildStatic) + "\n";
 txtOut += "    Install prefix: " + buildPrefix + "\n";
